/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     | Website:  https://openfoam.org
    \\  /    A nd           | Copyright (C) 2019 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "ParticleResponseTime.H"

// * * * * * * * * * * * * * Protected Member Functions  * * * * * * * * * * //

template<class CloudType>
void Foam::ParticleResponseTime<CloudType>::write()
{
    // Particle Reynolds Number
    IOField<scalar> Rep
    (
        this->owner().fieldIOobject("Rep", IOobject::NO_READ),
        this->owner().size()
    );

    // Particle drag Coefficent times Rep - Sphere (Putnam, 1961)
    IOField<scalar> CdRep
    (
	this->owner().fieldIOobject("CdRep", IOobject::NO_READ),
	this->owner().size()
    );

    // Relative velocity between the particle and the carrier
    IOField<vector> URel
    (
        this->owner().fieldIOobject("URel", IOobject::NO_READ),
        this->owner().size()
    );
    // Particle Response Time
    IOField<scalar> taup
    (
        this->owner().fieldIOobject("taup", IOobject::NO_READ),
        this->owner().size()
    );


    // velocity of the carrier at particle position
    autoPtr<interpolation<vector>> UcInterp
    (
        interpolation<vector>::New
        (
            this->owner().solution().interpolationSchemes(),
            this->owner().U()
        )
    );
    // density of the carrier at particle position
    autoPtr<interpolation<scalar>> rhocInterp
    (
        interpolation<scalar>::New
        (
            this->owner().solution().interpolationSchemes(),
            this->owner().rho()
        )
    );
    // dynamic viscosity of the carrier at particle position
    autoPtr<interpolation<scalar>> mucInterp
    (
        interpolation<scalar>::New
        (
            this->owner().solution().interpolationSchemes(),
            this->owner().mu()
        )
    );

    scalar rhoc;
    scalar muc;

    label i = 0;
    forAllConstIter(typename CloudType, this->owner(), iter)
    {
        URel[i] = iter().U() - UcInterp->interpolate( iter().coordinates(), iter().currentTetIndices() );

	rhoc = rhocInterp->interpolate( iter().coordinates(), iter().currentTetIndices() );
	muc = mucInterp->interpolate( iter().coordinates(), iter().currentTetIndices() );

	Rep[i] = rhoc*mag(URel[i])*iter().d()/muc; //ReynoldsNumber

	//Drag Coefficient

    	if (Rep[i] > 1000.0)
    	{
        	CdRep[i] = 0.424*Rep[i];
    	}
    	else
    	{
        	CdRep[i] = 24.0*(1.0 + 1.0/6.0*pow(Rep[i], 2.0/3.0));
    	}

	taup[i] = iter().rho()*pow(iter().d(), 2)/(18*muc)*(24/CdRep[i]); //Particle Response Time

        ++ i;
    }

    taup.write(this->owner().size() > 0);
}


// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

template<class CloudType>
Foam::ParticleResponseTime<CloudType>::ParticleResponseTime
(
    const dictionary& dict,
    CloudType& owner,
    const word& modelName
)
:
    CloudFunctionObject<CloudType>(dict, owner, modelName, typeName)
{}


template<class CloudType>
Foam::ParticleResponseTime<CloudType>::ParticleResponseTime
(
    const ParticleResponseTime<CloudType>& ppm
)
:
    CloudFunctionObject<CloudType>(ppm)
{}


// * * * * * * * * * * * * * * * * Destructor  * * * * * * * * * * * * * * * //

template<class CloudType>
Foam::ParticleResponseTime<CloudType>::~ParticleResponseTime()
{}


// ************************************************************************* //
